#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLEN 128
#define MAXVAR 30
#define MAXTYP 20

#define MAXLENFNAM 128
#define EXTMAX 10

FILE *fspec;
FILE *fgdef;
FILE *fginp;
FILE *fgdmp;
char specfile[MAXLENFNAM];
char gdeffile[MAXLENFNAM];
char ginpfile[MAXLENFNAM];
char gdmpfile[MAXLENFNAM];


/*================*/
int process_cmd(int argc, char** argv)
{
  int i, j;

  if(argc<2)
    {
      fprintf(stderr, "Usage: %s <filename>\n", argv[0]);
      fprintf(stderr, "  filename should not include the extension\n");
      fprintf(stderr, "  The extension '.def' will be appended to the");
      fprintf(stderr, " input file name.\n");
      return -1;
    }
  i = strlen(argv[1]);
  if(i>(MAXLENFNAM-EXTMAX))
    {
      fprintf(stderr, "filename too long %d > %d\n", i, MAXLENFNAM-EXTMAX);
      return -1;
    }
  strncpy(specfile, argv[1], i);
  strcat(specfile, ".def");
  strncpy(gdeffile, argv[1], i);
  strcat(gdeffile, "_gdef.inc");
  strncpy(ginpfile, argv[1], i);
  strcat(ginpfile, "_ginp.inc");
  strncpy(gdmpfile, argv[1], i);
  strcat(gdmpfile, "_gdmp.inc");

  fprintf(stderr, "Input filename:      %s\n", specfile);
  fprintf(stderr, "Definition filename: %s\n", gdeffile);
  fprintf(stderr, "Input code filename: %s\n", ginpfile);
  fprintf(stderr, "GIdump filename:     %s\n", gdmpfile);

  fspec = fopen(specfile, "r");
  if(fspec == NULL)
    {
      fprintf(stderr, "Could not fopen(%s)\n", specfile);
      return -1;
    }
  fgdef = fopen(gdeffile, "w");
  if(fspec == NULL)
    {
      fprintf(stderr, "Could not fopen(%s)\n", gdeffile);
      return -1;
    }
  fginp = fopen(ginpfile, "w");
  if(fginp == NULL)
    {
      fprintf(stderr, "Could not fopen(%s)\n", ginpfile);
      return -1;
    }
  fgdmp = fopen(gdmpfile, "w");
  if(fgdmp == NULL)
    {
      fprintf(stderr, "Could not fopen(%s)\n", gdmpfile);
      return -1;
    }
    
  return 0;
}
/*-------------*/
int main(int argc, char** argv)
{
  char line[MAXLEN];
  char* varnam;
  char* typnam;
  char* comment;

  if(process_cmd(argc, argv)) return -1;

  fprintf(fgdef, "#include <stdio.h>\n");
  fprintf(fgdef, "#include <stdlib.h>\n");
  fprintf(fgdef, "#include <string.h>\n");
  //  fprintf(fgdef, "#include <math.h>\n");
  //  fprintf(fgdef, "#include <malloc.h>\n");
  fprintf(fgdef, "#include <iostream>\n");
  fprintf(fgdef, "#include <fstream>\n");
  fprintf(fgdef, "#include <string>\n\n");
  fprintf(fgdef, "std::string GIline;\n");
  fprintf(fgdef, "char giline[%d];\n", MAXLEN);
  fprintf(fgdef, "std::string GIfile;\n");
  fprintf(fgdef, "std::ifstream GIf;\n");
  fprintf(fgdef, "FILE* fgdmp;\n");
  fprintf(fgdef, "\n");

  fprintf(fginp, "void GIinput()\n{\n");
  fprintf(fginp, "  char* GIwhat;\n");
  fprintf(fginp, "  GIf.open(GIfile.c_str());\n");
  fprintf(fginp, "  while(!GIf.eof())\n");
  fprintf(fginp, "    {\n");
  fprintf(fginp, "      getline(GIf, GIline);\n");
  fprintf(fginp, "      if(GIline.size()>0)\n");
  fprintf(fginp, "        {\n");
  fprintf(fginp, "          if(GIline[0] != '#')\n");
  fprintf(fginp, "            {\n");
  fprintf(fginp, "              strncpy(giline, GIline.c_str(), %d);\n",
	  MAXLEN);
  fprintf(fginp, "              GIwhat = strtok(giline, \": \\t\");\n");

  fprintf(fgdmp, "int GIdump(void)\n{\n");
  fprintf(fgdmp, "  if(strcmp(gidump.c_str(), \"stdout\")==0)\n");
  fprintf(fgdmp, "    {\n");
  fprintf(fgdmp, "      fgdmp = stdout;\n");
  fprintf(fgdmp, "    } else {\n");
  fprintf(fgdmp, "      if(strcmp(gidump.c_str(), \"stderr\")==0)\n");
  fprintf(fgdmp, "        {\n");
  fprintf(fgdmp, "          fgdmp = stderr;\n");
  fprintf(fgdmp, "        } else {\n");  
  fprintf(fgdmp, "          fgdmp = fopen(gidump.c_str(), \"w\");\n");
  fprintf(fgdmp, "        }\n");  
  fprintf(fgdmp, "    }\n");  
  fprintf(fgdmp, "  if(fgdmp == NULL)\n    {\n");
  fprintf(fgdmp, "      std::cerr<<\"Could not fopen(\"<<gidump<<\")\\n\";\n");
  fprintf(fgdmp, "      return -1;\n    }\n");

  while(fgets(line, MAXLEN, fspec))
    {
      typnam = strtok(line, ":, \t");
      varnam = strtok(NULL, ":, \t\n");
      comment = strtok(NULL, "");
      if(typnam[0]=='#')continue;  // Skip comments
      if(strcmp(typnam, "string")==0) typnam = "std::string";
      if(comment!=NULL)
	{
	  fprintf(fgdef, "%s\t%s;\t//%s", typnam, varnam, comment);
	} else {
	  fprintf(fgdef, "%s\t%s;\n", typnam, varnam);
	}

      fprintf(fginp, "              if(strcmp(GIwhat, \"%s\")==0)\n", varnam);
      fprintf(fginp, "                {\n");
      fprintf(fginp, "                  GIwhat = strtok(NULL, \": \\t\");\n");
      if(strcmp(typnam, "int")==0)
	{
	  fprintf(fginp, "                  %s = atoi(GIwhat);\n", varnam);
	  fprintf(fgdmp, "  fprintf(fgdmp, \"#%s:\\t%cd\\n\", %s);\n",
		  varnam, '%',  varnam);
	}
      if(strcmp(typnam, "double")==0)
	{
	  fprintf(fginp, "                  %s = atof(GIwhat);\n", varnam);
	  fprintf(fgdmp, "  fprintf(fgdmp, \"#%s:\\t%cf\\n\", %s);\n",
		  varnam, '%',  varnam);
	}
      if(strcmp(typnam, "std::string")==0)
	{
	  fprintf(fginp, "                  %s = GIwhat;\n", varnam);
	  fprintf(fgdmp, "  fprintf(fgdmp, \"#%s:\\t%cs\\n\", %s.c_str());\n",
		  varnam, '%',  varnam);
	}

      fprintf(fginp, "                }\n");
    }
  fprintf(fginp, "         }\n");
  fprintf(fginp, "       }\n");
  fprintf(fginp, "     }\n");
  fprintf(fginp, "  GIf.close();\n}\n");

  fprintf(fgdmp, "  return 0;\n");
  fprintf(fgdmp, "}\n");
  
  fprintf(fgdef, "\n");
  fprintf(fgdef, "#include \"%s\"\n", ginpfile);
  fprintf(fgdef, "#include \"%s\"\n", gdmpfile);
  
  return 0;
}
