// $Header: /home/ra/ride/rap/lambda/Geom3d/RCS/Line_test.cxx,v 1.1 1996/02/23 02:02:08 mackay Exp mackay $
// This program tests some of the Line class functions defined in Geom3d.h,

#include "Geom3d.hxx"

//==============================
int main(int argc, char **argv)
{
  Point p1(0.,1.,0.), p2(0,0,1.);
  Point orig(0,0,0);


  std::cout.precision(8);
  std::cout.setf(std::ios::fixed);


  std::cout << "p1: " << p1 << "\n";
  std::cout << "p2: " << p2 << "\n";

  std::cout << "Test Line constructors\n";
  Line  l1;
  std::cout << "Default line: " << l1 << "\n";
  Line  l2(p1,p2);
  std::cout << "line through p1 & p2   l2 =" << l2 << "\n";
  Line  l3(Point(0,0,0),1,1,0);  // anchor and 3 unnormalized dir cosines.
  std::cout << "l3: " << l3 << "\n\n";

  std::cout << "l2.anchor():" << l2.anchor() << "\n";
  std::cout << "l2.anchor().[x, y, z]:"
       << l2.anchor().x
       << l2.anchor().y
       << l2.anchor().z << "\n";

  l1.Setanchor(p1);
  std::cout << "l1.Setanchor(p1): " << l1 << "\n";
  l1.Setx(2.0);  l1.Sety(2.0);  l1.Setz(2.0);
  std::cout << "l1.Setx(2); l1.sety(2); l1.setz(2):" << l1 << "\n";
  std::cout << "l1.cosines() = " << l1.cosines() << "\n";
  l1.Setcosines(p1);
  std::cout << "l1.Setcosines(p1): " << l1 << "\n";
  l1.Setcosines(1,1,1);
  std::cout << "l1.Setcosines(1,1,1): " << l1 << "\n";

//  translations etc
  std::cout << "\n" << "p1: " << p1 << "\n";
  std::cout << "l2: " << l2 << "\n";

  std::cout << "l2+p1:" << l2+p1 << "\n";
  std::cout << "p1+l2:" << p1+l2 << "\n";
  std::cout << "l2-p1:" << l2-p1 << "\n";
  std::cout << "p1-l2:" << p1-l2 << "\n";

  l2 += p1;
  std::cout << "l2+=p1: " << l2 << "\n";
  l2 -= p1;
  std::cout << "l2-=p1: " << l2 << "\n";
  std::cout << "-l2: " << -l2 << "\n";

  std::cout <<   "(l1 || l1):" << (l1 || l1)
       << "\t(l1 || l2):" << (l1 || l2) << "\n";
  Line lx(orig,1,0,0), ly(orig,0,1,0);

  std::cout << "\n" << "lx:" << lx << "\n";
  std::cout <<         "ly:" << ly << "\n";
  std::cout <<   "perp(lx,lx):" << perp(lx,lx)
       << "\tperp(lx,ly):" << perp(lx,ly) << "\n";

// projection, intersection distance operations:
  std::cout << "\n projection, intersection, & distance functions:\n";

  std::cout << "l2.project(p1): " << l2.project(p1) << "\n";
  std::cout << "angle(lx,ly): " << angle(lx,ly) << "\n";
  std::cout << "angle(l1,l2): " << angle(l1,l2) << "\n";

  double d;
  std::cout << "intersect(l1,l2):" << intersect(l1,l2,&d)
       << " dist=" << d << "\n";

  std::cout << "distance(p1,l1): " << distance(p1,l1) << "\n";
  std::cout << "distance(l1,p1): " << distance(l1,p1) << "\n";

// rotations
  double Piover4 = Piover2/2;

  l1 = Line(Point(1,1,1),1,0,0);
  std::cout << "l1: " << l1 << "\n";

  l1 = rotate_x(l1,Piover4);
  std::cout << "l1=rotate_x(l1,Pi/4): "  << l1 << "\n";

  l1 = rotate_z(l1,-Piover2);
  std::cout << "l1=rotate_z(l1,-Pi/2): " << l1 << "\n";

  l1 = rotate_y(l1,Piover4);
  std::cout << "l1=rotate_y(l1,Pi/4): "  << l1 << "\n";

  l1 = rotate_z(l1,Piover2);
  std::cout << "l1=rotate_z(l1,Pi/2): "  << l1 << "\n";

// Try the fitline routine.
  const int npts = 10;
  Point q[npts];
  int i;
  for(i=0;i<npts;i++)
    {
      q[i].x = i;
      q[i].y = 3*i+0.01*sin(i);
      q[i].z = 2*i+0.01*cos(i);
    }
  Line lfit = fitline(q,npts);
  std::cout << "\nFit a line through the following points:\n";
  for(i=0;i<npts;i++)
    {
      std::cout << i << ":\t" << q[i] << "\tresid:"
	   << q[i]-lfit.project(q[i]) << "\n";
    }
  std::cout << "The fitted line is:" << lfit << "\n";

  return 0;
}
