// sixvec.hxx
#ifndef SIXVEC_HXX
#define SIXVEC_HXX 1

/* This is a class of 6-d vectors consisting of two 3-d vectors:
      a spacial coordinate X
      a momentum coordinate P
   At present only simple arithmetic operators are defined:
      C=A+B;
      C=A-B;
      C=A*r;
      C=r*A;
      C=A/r;
      C+=A;
      C-=A;
      C*=r;
      C/=r;
   where A, B, and C are sixvec's and r is a double.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <iostream>
#include <fstream>
#include <Geom3d/Geom3d.hxx>

//===========================
class sixvec
{
public:
  Point X; // spatial coords
  Point P; // momentum coords
// Constructors:
  //-----------------
  sixvec(void) : X(Point(0,0,0)), P(Point(0,0,0)) {};
  //-----------------
  sixvec(Point x, Point p) :X(x), P(p) {};
  //-----------------
  sixvec(double x, double y, double z, double px, double py, double pz)
    : X(Point(x,y,z)), P(Point(px, py, pz)) {};
// Operators:
  //-----------------
  sixvec operator+(const sixvec &A) const
  {
    sixvec C;
    C.X = X+A.X;
    C.P = P+A.P;
    return C;
  };
  //-----------------
  sixvec operator-(const sixvec &A) const
  {
    sixvec C;
    C.X = X-A.X;
    C.P = P-A.P;
    return C;
  };
  //-----------------
  sixvec operator+=(const sixvec &A)
  {
    this->X += A.X;
    this->P += A.P;
    return *this;
  }
  //-----------------
  sixvec operator-=(const sixvec &A)
  {
    this->X -= A.X;
    this->P -= A.P;
    return *this;
  }
  //-----------------
  sixvec operator*(const double r) const
  {
    sixvec C;
    C.X = X*r;
    C.P = P*r;
    return C;
  };
  //-----------------
  sixvec operator*=(double r)
  {
    this->X *= r;
    this->P *= r;
    return *this;
  }
  //-----------------
  sixvec operator/(const double r) const
  {
    sixvec C;
    C.X = X/r;
    C.P = P/r;
    return C;
  };
  //-----------------
  sixvec operator/=(double r)
  {
    this->X /= r;
    this->P /= r;
    return *this;
  }
  //=============
  friend sixvec operator*(const double r, const sixvec &V)
  {
    sixvec C;
    C = V*r;
    return C;
  };
  //-----------------
  friend sixvec operator-(const sixvec A)
  { // negation operator
    sixvec t;
    t.X = -A.X;
    t.P = -A.P;
    return t;
  };
  //-----------------
  friend std::ostream &operator<<(std::ostream &os, const sixvec &A)
  {
    os << "[ "
       << A.X.x << ", " << A.X.y << ", " << A.X.z << ", "
       << A.P.x << ", " << A.P.y << ", " << A.P.z << " ]";
    return os;
  }
};
//========================
#endif
