// $Header: $
// mat3.cxx

#include <iostream>
#include <stdio.h>
#include <stdlib.h>

#define mat3_lib_goober 1
#include "mat3/mat3.hxx"

// Functions for "mat3" class

//=====================
mat3 mat3::unit()
{
  int i, j;
  for(i=0;i<MAT3DIM;i++)
    {
      for(j=0;j<MAT3DIM;j++)
	{
	if(i==j)m[i][j] = 1;
	else m[i][j] = 0;
	}
    }
  return *this;
}
//-- -- -- -- -- --
mat3 mat3::zero()
{
  int i, j;
  for(i=0;i<MAT3DIM;i++)
    {
      for(j=0;j<MAT3DIM;j++)
	{
	  m[i][j] = 0;
	}
    }
  return *this;
}
//-- -- -- -- -- --
mat3 &mat3::operator=(const mat3 &A)
{
  int i,j;

  for(i=0;i<MAT3DIM;i++)
    {
      for(j=0;j<MAT3DIM;j++)
        {
          m[i][j] = A.m[i][j];
        }
    }
  return *this;
}
//-- -- -- -- -- --
mat3 mat3::operator*(const mat3 &A) const
{
  int i,j,k;
  mat3 C;

  for(i=0;i<MAT3DIM;i++)
    {
      for(j=0;j<MAT3DIM;j++)
        {
          C.m[i][j] = 0;
          for(k=0;k<MAT3DIM;k++)
            {
              C.m[i][j] += m[i][k] * A.m[k][j];
            }
        }
    }
  return C;
}
//-- -- -- -- -- --
mat3 mat3::operator+(const mat3 &A) const
{
  int i,j;
  mat3 C;

  for(i=0;i<MAT3DIM;i++)
    {
      for(j=0;j<MAT3DIM;j++)
        {
          C.m[i][j] = m[i][j]+A.m[i][j];
        }
    }
  return C;
}
//-- -- -- -- -- --
mat3 mat3::operator-(const mat3 &A) const
{
  int i,j;
  mat3 C;

  for(i=0;i<MAT3DIM;i++)
    {
      for(j=0;j<MAT3DIM;j++)
        {
          C.m[i][j] = m[i][j]-A.m[i][j];
        }
    }
  return C;
}
//-- -- -- -- -- --
std::ostream &operator<<(std::ostream & os, const mat3 &A)
{
  int i,j;
  char numstr[50];

  for(i=0;i<MAT3DIM;i++)
    {
      os << "\t[ ";
      for(j=0;j<MAT3DIM;j++)
        {
	  //          sprintf(numstr, (char*)mat3_format, A.m[i][j]);
          sprintf(numstr, "%11.6lf", A.m[i][j]);
          os << numstr;
        }
      os << " ]\n";
    }
  return os;
}
//-- -- -- -- -- --
mat3 transpose(const mat3 &M)
{
  int i,j;
  mat3 A;
  for(i=0;i<MAT3DIM;i++)
    {
      for(j=0;j<MAT3DIM;j++)
	{
	  A.m[i][j] = M.m[j][i];
	}
    }
  return A;
}
//-- -- -- -- -- --
double trace(const mat3 &M)
{
  int i;
  double t;
  t = 0;
  for(i=0;i<MAT3DIM;i++)
    {
      t += M.m[i][i];
    }
  return t;
}
//-- -- -- -- -- --
double det(const mat3 &M)
{
  return M.m[0][0]*(M.m[1][1]*M.m[2][2]-M.m[2][1]*M.m[1][2])
    +M.m[1][0]*(M.m[0][2]*M.m[2][1]-M.m[0][1]*M.m[2][2])
    +M.m[2][0]*(M.m[0][1]*M.m[1][2]-M.m[0][2]*M.m[1][1]);
}
//-- -- -- -- -- --
mat3 inverse(const mat3 &M)
{
  double d = det(M);
  
  if(d==0)
    {
      std::cerr
	<< "? Attempt to take inverse of mat3 with zero determinant.\n";
      return M;
    }

  mat3 t;

  t.m[0][0] = (M.m[1][1]*M.m[2][2]-M.m[2][1]*M.m[1][2])/d;
  t.m[1][0] = (M.m[1][2]*M.m[2][0]-M.m[1][0]*M.m[2][2])/d;
  t.m[2][0] = (M.m[1][0]*M.m[2][1]-M.m[2][0]*M.m[1][1])/d;

  t.m[0][1] = (M.m[0][2]*M.m[2][1]-M.m[0][1]*M.m[2][2])/d;
  t.m[1][1] = (M.m[0][0]*M.m[2][2]-M.m[0][2]*M.m[2][0])/d;
  t.m[2][1] = (M.m[2][0]*M.m[0][1]-M.m[2][1]*M.m[0][0])/d;

  t.m[0][2] = (M.m[0][1]*M.m[1][2]-M.m[1][1]*M.m[0][2])/d;
  t.m[1][2] = (M.m[1][0]*M.m[0][2]-M.m[0][0]*M.m[1][2])/d;
  t.m[2][2] = (M.m[0][0]*M.m[1][1]-M.m[1][0]*M.m[0][1])/d;

  return t;
}
