// $Header: $
// mat6.hxx  a 4x4 matrix class

#ifndef mat6_h
#define mat6_h 1

#include <iostream>
#include <math.h>

#ifndef mat6_lib_goober
//extern wstring mat6_format;
#else
//wstring mat6_format = " %11.6lf";
#endif

const int MAT6DIM = 6;

//=====================
class mat6	// a matrix class
{
public:
  double m[MAT6DIM][MAT6DIM];
//-- -- -- -- -- -- --
// constructors:
  mat6()
    {
      unit();  // default to a unit matrix
    }

// Member functions, etc.
  mat6 unit();
  mat6 zero();
  mat6 &operator=(const mat6 &A);
  mat6 operator*(const mat6 &A) const;
  mat6 operator+(const mat6 &A) const;
  mat6 operator-(const mat6 &A) const;
  friend std::ostream &operator<<(std::ostream & os, const mat6 &m);
  friend mat6 transpose(const mat6 &M);
  friend double trace(const mat6 &M);
  friend mat6 operator*(double s, const mat6 &M)
  {
    mat6 t;
    int i, j;

    for(i=0;i<MAT6DIM;i++)
      for(j=0;j<MAT6DIM;j++)
	t.m[i][j] = s*M.m[i][j];
    return t;
  }
  friend mat6 exp6(mat6 M, int n);
  friend mat6 inverse(const mat6 &A);
  friend mat6 rot6x(const double &t)
  {
    mat6 R;
    R.m[2][2] = R.m[3][3] = R.m[4][4] = R.m[5][5] = cos(t);
    R.m[2][4] = R.m[3][5] = sin(t);
    R.m[4][2] = R.m[5][3] = -R.m[2][4];
    return R;
  }
  friend mat6 rot6y(const double &t)
  {
    mat6 R;
    R.m[0][0] = R.m[1][1] = R.m[4][4] = R.m[5][5] = cos(t);
    R.m[0][4] = R.m[1][5] = -sin(t);
    R.m[4][0] = R.m[5][1] = -R.m[0][4];
    return R;
  }
  friend mat6 rot6z(const double &t)
  {
    mat6 R;
    R.m[0][0] = R.m[1][1] = R.m[2][2] = R.m[3][3] = cos(t);
    R.m[0][2] = R.m[1][3] = sin(t);
    R.m[2][0] = R.m[3][1] = -R.m[0][2];
    return R;
  }
};
#endif
