//---------------
void dumpspinor(const int i, const su2spinor Psi, const Trajectory X)
{
  const char fmt1[]="%-15s\t%14.9f";
  const char fmt2[]=",\t%14.9f\t%14.9f";
  const char fmt3[]=",\t%16.12f\t%16.12f\t%16.12f";
  const char fmt4[]=",\t%14.9f\t%14.9f\t%14.9f\t%14.9f";
  const char fmt6[]=",\t%16.12f\t%16.12f\t%16.12f\t%16.12f\t%16.12f\t%16.12f";
  char line[150];
  std::string name;

  if(skipflg>0) return;

  Trajectory XX = lat[i].M*X;
  Point n = vector(Psi);
  name = lat[i].SWN+",";
  sprintf(line, fmt1, name.c_str(), lat[i].sdn);
  fspinor<<line;
  sprintf(line, fmt3, n.x, n.y, n.z);
  fspinor<<line;
  sprintf(line, fmt4, real(Psi.p[0]), imag(Psi.p[0]),
	  real(Psi.p[1]), imag(Psi.p[1]));
  fspinor<<line;
  sprintf(line, fmt6, XX.v[0], XX.v[1], XX.v[2], XX.v[3], XX.v[4], XX.v[5]);
  fspinor<<line;
  fspinor<<endl;
  fspinor.flush();
}
//----------------
su2mat Sdip(const int i, const Trajectory Xin)
{ 
  su2mat Px, Py, Pz, Pt, DR, R;  // Class initializes to unit matrices.
  int j;
  double theta  = lat[i].par[0];
  double dtheta = theta/((double)nbsteps);
  totang0 += theta;
  double rho    = lat[i].l/theta;
  double dlen   = lat[i].l/((double)nbsteps);
  Trajectory X1, X0, dX, Xh;
  X0 = Xin;
  double tmp = 1./(1.0+X0.v[5]); // momentum correction
  double den0, den1;
  double tx, ty;
  Tfunc Mslice;

  if(fabs(theta)<=zerotol) return R;

  if(nbsteps>1)
    Mslice = Sbend(dlen, dtheta, 0, 0, 0, 0, 1, beta);
  else
    Mslice = lat[i].M;

  //  Pt = Ry(Gg*tmp*dtheta/2.);
  Pt = Ry(Gg*dtheta/2.);

  for(j=0;j<nbsteps;j++)
    {
      X1 = Mslice*X0;
      dX = X1-X0;
      Xh = 0.5*(X0+X1);

      den0 = sqrt((1.0+X0.v[5])*(1.0+X0.v[5])-X0.v[1]*X0.v[1]-X0.v[3]*X0.v[3]);
      den1 = sqrt((1.0+X1.v[5])*(1.0+X1.v[5])-X1.v[1]*X1.v[1]-X1.v[3]*X1.v[3]);
      tx = -(atan(X1.v[1]/den1)-atan(X0.v[1]/den0));
      ty =   atan(X1.v[3]/den1)-atan(X0.v[3]/den0);

      if(abs(ty)>1.0e-12)
	{
	  std::cerr<< "Sdip: ty = "<<ty<<" !=0\n";
	  exit(-1);  // This is just to check.
	}
      totangdt0 += dtheta;
      totangh += tx;
      totangv += ty;
      //      Px = Rx(Ggp1*tmp*ty/2.0*dtheta);  // Worng
      //      Py = Ry(Ggp1*tmp*tx/2.0*dtheta);  // Worng
      //      Pz = Rz(-Ggm1*tmp*Xh.v[3]*dtheta);

// First order in trajectory coordinates:
      //      Px = Rx(Ggp1*ty/2.0);    //  I expect ty=0 in dipole
      //      Py = Ry(Ggp1*tx/2.0);
      Py = Ry(Ggp1*tx/2.0);
      Pz = Rz(-Ggm1*Xh.v[3]*dtheta);

      DR = Pt*Px*Py*Pz*Py*Px*Pt;
      X0 = X1;
      R = DR*R;
    }
  /*
  fprintf(ftmp, "%12.9f  %12.9f  %12.9f  %12.9f  %12.9f  %12.9f  ",
  	  lat[i].l, theta, dtheta, theta/dtheta, totang0, totangdt0);
  fprintf(ftmp, "%12.9f  %12.9f    %s\n",
	  totangh, totangv, lat[i].SWN.c_str());
  */
  return R;
}
//---------------
su2mat Squad(const int i, const Trajectory Xin)
{
  int j;
  su2mat Px, Py, Pz, DR, R;
  double tx, ty;
  Trajectory X0, X1, dX;
  X0 = Xin;
  double tmp = 1./(1.0+X0.v[5]);
  double k = lat[i].par[1];
  double dlen = lat[i].l/((double)nqsteps);

  X1 = lat[i].M*X0;

  Tfunc Mslice;
  if(nqsteps>1)
    Mslice = Quad(dlen, k, lat[i].par[3], 1, beta);
  else
    Mslice = lat[i].M;

  //Trajectory Xh;
  //  Xh = 0.5*(X0+X1);

  for(j=0;j<nqsteps;j++)
    {
      X1 = Mslice*X0;
      //      dX = X1-X0;
      //      tx = -atan(dX.v[1]);
      //      ty =  atan(dX.v[3]);
      tx = -(atan(X1.v[1])-atan(X0.v[1]));
      ty =   atan(X1.v[3])-atan(X0.v[3]);
      totangh += tx;
      totangv += ty;
      Px = Rx(Ggp1*tmp*ty/2.0);
      Py = Ry(Ggp1*tmp*tx/2.0);
      //Pz = ?  // ignore second order contribution for now
      DR = Px*Py*Pz*Py*Px;
      X0 = X1;
      R = DR*R;
    }
  return R;

}
//---------------
su2mat Sstraight(const int i, const Trajectory Xin)
{ 
  su2mat Px, Py, Pz;
  double tx, ty;
  Trajectory X0, X1, dX;
  X0 = Xin;
  X1 = lat[i].M*X0;
  double tmp = 1./(1.0+X0.v[5]);

  //Trajectory Xh;
  //  Xh = 0.5*(X0+X1);

  //  dX = X1-X0;
  //  tx = -atan(dX.v[1]);
  //  ty =  atan(dX.v[3]);
  tx = -(atan(X1.v[1])-atan(X0.v[1]));
  ty =   atan(X1.v[3])-atan(X0.v[3]);
  totangh += tx;
  totangv += ty;
  Px = Rx(Ggp1*tmp*ty/2.0);
  Py = Ry(Ggp1*tmp*tx/2.0);
  //Pz = ?  // ignore second order contribution for now
  return Px*Py*Pz*Py*Px;
}
//---------------
su2mat Ssolen(const double length, const double k, const Trajectory X)
{  // this is only approximately correct
  su2mat Pbody, Px, Py, Pz;
  Point ne;
  double mu;
  double norm;
  Tfunc Mhs;
  Trajectory Xh, X1;
  double xp, yp;

  // Definition from 2nd Ed. of  C&M Eq 4.66:
  //      k = Bsol/rigidity = Bsol * q / p.
  if(k==0.0)
    return Pbody; // identity

  Mhs = Solen(length/2., k, beta);
  Xh = Mhs*X;
  //  X1 = Mhs*Xh;
  xp = Xh.v[1]+k*length/2.*Xh.v[2]; // x' = (w_x + qBsol/2/p *y)
  yp = Xh.v[3]-k*length/2.*Xh.v[0]; // x' = (w_x + qBsol/2/p *y)

  Pz = Rz(Gp1*k*length/2.0);
  Px = Rx(-Ggm1*xp*k/2.0);
  Py = Ry(-Ggm1*yp*k);
  Pbody = Pz*Px*Py*Px*Pz;

  return Pbody;
}
//---------------
su2mat Ssextipole(int i, Trajectory *X)
{
  su2mat Px, Py, Pz;
  double tx, ty;
  Trajectory X0, X1, Xm, Xtmp;
  double tmp = 1./(1.0+X0.v[5]);
  double len = lat[i].l;
  double k2  = lat[i].par[2];
  //  double tilt = lat[i].par[3];   // Ignore for now
  X0 = *X;
  Xtmp = Drift(len, beta)*X0;
  Xm = Xtmp;
  Xm.v[1] = Xm.v[1]+0.5*k2*len * (Xtmp.v[0]*Xtmp.v[0]-Xtmp.v[2]*Xtmp.v[2]);
  Xm.v[3] = Xm.v[3]+    k2*len * Xtmp.v[0]*Xtmp.v[2];
  X1 = Drift(len, beta)*Xm;
  *X = X1;

  tx = -(atan(X1.v[1])-atan(X0.v[1]));
  ty =   atan(X1.v[3])-atan(X0.v[3]);
  totangh += tx;
  totangv += ty;
  Px = Rx(Ggp1*tmp*ty/2.0);
  Py = Ry(Ggp1*tmp*tx/2.0);
  //Pz = ?  // ignore second order contribution for now
  return Px*Py*Pz*Py*Px;
  
}
//---------------
su2mat Smultipole(int i, Trajectory *X)
{
  int j;
  su2mat Px, Py, Pz;

  return Px;  // return an identity for now

  double tx, ty;
  Trajectory X0, X1;
  double tmp = 1./(1.0+X0.v[5]);
  double len = lat[i].l;
  double kl[4], tilt[4];
  kl[0] = lat[i].par[0];
  kl[1] = lat[i].par[1];
  kl[2] = lat[i].par[2];
  kl[3] = lat[i].par[4];  // Why did MAD use this ordering?
  tilt[0] = lat[i].par[3];
  tilt[1] = lat[i].par[5];
  tilt[2] = lat[i].par[6];
  tilt[3] = lat[i].par[7];
  // For now only deal with a thin-lens normal sextipole.
  if( (tilt[0] != 0) || (tilt[1] != 0) || (tilt[2] != 0) || (tilt[3] != 0) )
    {
      std::cerr<<"Smultipole: Found a tilted multipole!\t"<<lat[i].SWN<<"\n";
      for(j=0;j<4;j++)
	std::cerr<<"t:";
	std::cerr<<"\t"<<tilt[j];
      exit(-1);
    }
  if( (kl[0] != 0) || (kl[1] != 0) || (kl[3] != 0) )
    {
      std::cerr<<"Smultipole: Found a nonsextipole multipole!\t"
	       <<lat[i].SWN<<"\n";
      std::cerr<<"kl:";
      for(j=0;j<4;j++)
	std::cerr<<"\t"<<kl[j];
      exit(-1);
    }
  X0 = *X;
  X1 = X0;
  X1.v[1] = X1.v[1]+0.5*kl[2] * (X0.v[0]*X0.v[0]-X0.v[2]*X0.v[2]);
  X1.v[3] = X1.v[3]+    kl[2] * X0.v[0]*X0.v[2];
  *X = X1;

  tx = -(atan(X1.v[1])-atan(X0.v[1]));
  ty =   atan(X1.v[3])-atan(X0.v[3]);
  totangh += tx;
  totangv += ty;
  Px = Rx(Ggp1*tmp*ty/2.0);
  Py = Ry(Ggp1*tmp*tx/2.0);
  //Pz = ?  // ignore second order contribution for now
  return Px*Py*Pz*Py*Px;
}
//---------------
//su2mat Sth(constant double len, constant double theta, const Trajectory X);
//   use the dipole with tilt=0, and -theta
//---------------
//su2mat Stv(constant double len, constant double theta, const Trajectory X);
//   use the dipole with tilt=pi/2.
//---------------
